// Global Header file of the CForth  project **********************************
//
// Author and (c)2020-24 Wolfgang Schemmert. The copyright claim is restricted
// to this file and the depending files "main.c", "main.h", "cforth.c",
// "cforth.h", "usb-otg-ctrlbd.c" and "dmx.c"
// Other depending material is published, copyrighted and licensed
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#ifndef GLOBALDEFINES
#define GLOBALDEFINES

#define REVNUM  3			//describes firmware revision

#define BLACKPILL

#ifdef BLACKPILL
//#define DMX					//optional simple MDX512 transmitter
#define HARDWARE_VERSION 3  //describes project specific hardware version
							//new design expansion PCB: 11 Feb 2024
							//User(Front)LED changed to PB15
#else	//NUCLEO64 versions


#define STLINK_CUTOFF

  #ifdef STLINK_CUTOFF
#define HARDWARE_VERSION 3	//describes project specific hardware version
							//new design 19 Nov 2021
//#define DMX					//optional simple DMX512 transmitter
  #else
#define ORIG_NUCLEO
#define HARDWARE_VERSION 1	//unmodified
//the onboard ST-Link module is cut off or not used
// if NOT defined:
// unmodified Nucleo-F411
// then STM32F411 internal USB is NOT active
// and communication RESTRICTED TO USART2 (driven by ST-LINK module)
  #endif
#endif	//BLACKPILL or NUCLEO

// *****************************

//the device will appear as "STMicroelectronics Virtual Com Port"
#define DEFAULT_VIDH	0x04
#define DEFAULT_VIDL	0x83
#define DEFAULT_VID		((DEFAULT_VIDH << 8) | DEFAULT_VIDL)
#define DEFAULT_PIDH	0x57
#define DEFAULT_PIDL	0x40
#define MIDI_PLUS	3	//added to actual VID/PID in USB is MIDI interface
#define DEFAULT_PID		((DEFAULT_PIDH << 8) | DEFAULT_PIDL)

#define DEFAULT_NUMBASE	10; //Decimal, enter 16 for HEX
#define DEFAULT_BAUD	11

//attributes of KernelOperators
//attribute entries are merged with some other functional flags !!
#define A_IME			0x10000 //ATTRIBUTE_IMMEDIATE, cannot be compiled
#define A_CPO			0x20000 //ATTRIBUTE_COMPILEONLY,refuse work interpreting
#define A_BAK			0x40000 //may be parsed in background at executing state
#define A_CI			(A_CPO | A_IME) //COMPILEONLY IMMEDIATE
								//performs complex operations during compilation
#define A_IB			(A_IME | A_BAK)
#define A_RUN			0x80000 //ATTRIBUTE_RUNTIME,
					//compiled by other operators for special runtime behaviour
					//no direct user access, essential for viewing SEE and STEP
//only attribute of UserToken
#define A_VARCON		0x0C000000

#define ERR_STRINGTOKEN 1
#define ERR_SYNTAX		2
#define ERR_STACK		4
#define ERR_COMPONLY	8
#define ERR_SYSTEM		0x10
#define ERR_USERCODE	0x20
#define ERR_COMMACC		0x40
#define ERR_CONTEXT 	0x80
#define ERR_USERCONTEXT	0xA0	//combination of both, save code memory
#define ERR_USERSTRING	0x21	//combination of both, save code memory
#define ERR_CANCEL		0x100
#define ERR_RUNTOKEN	0x200
#define ERR_NUMVAL		0x400
#define ERR_DIVZERO		0x800

#ifndef ORIG_NUCLEO
  #ifdef DMX
#define KERNELSIZE      166
#define DMXFIRST		146
  #else
#define KERNELSIZE      147
   #endif
#else
#define KERNELSIZE      137
#endif

#define MAXUSER			168//*6 words p.struct=1008 words=4032 bytes=ca.2pages
#define MAXSTACK		64
#define MAXCODE			0xC00   // =3072 words =12288 bytes
                                //takes 6 pages each 2048 bytes
#define CODETOP         MAXCODE - 1
#define USERTOP         (uint32_t)&gfUserHeader[MAXUSER]

#define CDC_CLASS   0       //initalizes device as virtual COM port
#define MIDI_CLASS  1       //initalizes device as native MIDI interface
 
#define USART_RXBUFLEN	4096
#define MIDI_BUFLEN		 256
#define USB_BUFLEN		4096
#define DMX_BUFLEN		512 // index 0 is DMX addr. 1, TX from index 0 to 512

#define CTRL_B  2
#define CTRL_C  3
#define BKSP  8
#define TAB   9
#define LFF  10		//for Macintosh OS: simply exchange values of CRR and LFF
#define CRR  13		//then New_Line will be interpreted correctly
#define CTRL_R  18
#define ESC  27
#define SPACE  32
#define BACKSLASH 92

//not relevant for this project, but necessary for compilation:
#define EOX     0xF7
#define RUNNINGSTATE  //uncomment if not wanted
//works only on MIDI msgs which are transmitted from MIDI OUT
//Running State is always supported with MIDI msgs received at MIDI IN

//bitmask shortcuts:
#define mask0      1
#define mask1      (1<<1)
#define mask2      (1<<2)
#define mask3      (1<<3)
#define mask4      (1<<4)
#define mask5      (1<<5)
#define mask6      (1<<6)
#define mask7      (1<<7)
#define mask8      (1<<8)
#define mask9      (1<<9)
#define mask10     (1<<10)
#define mask11     (1<<11)
#define mask12     (1<<12)
#define mask13     (1<<13)
#define mask14     (1<<14)
#define mask15     (1<<15)
#define mask16     (1<<16)
#define mask17     (1<<17)
#define mask18     (1<<18)
#define mask19     (1<<19)
#define mask20     (1<<20)
#define mask21     (1<<21)
#define mask22     (1<<22)
#define mask23     (1<<23)
#define mask24     (1<<24)
#define mask25     (1<<25)
#define mask26     (1<<26)
#define mask27     (1<<27)
#define mask28     (1<<28)
#define mask29     (1<<29)
#define mask30     (1<<30)
#define mask31     (1<<31)
#endif

// ##########################################################################
// Version changes:
//REVNUM 1: 23Jan24, combined from code for STM32L476 state 19Jan24
//and older USB-Nucleo-F411 EmBitz project
//handware for STLINK_CUTOFF = hardware for STM32L476 plus USART_TX inverter
//23Jan24	first working version with unmodified STM32F411 Nucleo
//09Feb24	first completely working version w. STM32F411 Nucleo STLINK_CUTOFF
//12Feb24	DMX packet transferred from STM32G4
//17Feb24	first completely working version with STM32F411 BlackPill
//26Feb24	program flow IADRXN improved
//07March24	kernel op RADC: only 1 single sample used for readout
//08March24	kernel op RADC: alternative only 4 sample average used for readout
//09March24 flash storage of gdFadeTime and gfBootScene simplified & improved
//			reorg kernel op SAVE and LOAD more Forth typically
//			I2C stability improved (I2C_SR1_AF bit checked)
//12March24	I2C code made more failure-resistant
//released 14March2024
//REVNUM->2
//31March24	bugfix cforth.h: DZ and HX have IMMEDIATE flag now
//02Apr24	doVALUE renamed doVAL
//released 09April2024
//REVNUM->3
//10Apr24	op_MOT1,2,3 modified: NSTP1,2,3=-2-->permanent rotation
//20Apr24	doLOOP() end index behaviour changed
//06Sept24	BlackPill User(Front)LED changed to PB15
//28Dec24	FlashSector3: order gfNumbase, gBaud exchanged
//29Dec24	subPWMx() reorganized
//			checkSpecialIO() renamed checkPWM()
//			{"REV"} renamed {"WVA"}
//04Jan25	op_LED()
//			servSystem(): LED blink behaviour changed for better user access
//			op_SPI() simplified
//16Jan25	{"REV"} renamed {"W"}.Previous kernel operators "W" and "R"removed
//			op_DOREV() renamed op_WVA()
//19Jan25	VARCONs directly stored in user code. op_VALUE unnecessary/removed
//20Jan25	executeTCode() speed optimization
//25Jan25	bugfix op_READDIG: wrong pinNo filter mask
//26Jan25	debug state displayed in op_QUEST.
//			Bugfix  executeTCode() and op_ABORT, NO_DBG was always cancelled
//29Jan25	executeTCode(): termination code modified
//01Feb25	DEBUG renamed BACKOP, NO_DBG renamed NOBACK
//			CTRL_B->CTRL_C, CTRL_D->CTRL_B
//07Feb25	XBUS ,gfDmxBusy introduced: speed up kernel when DMX is not needed

// *****************************
