// CForth  project ************************************************************
//
// Author and (c)2020-25 Wolfgang Schemmert. The copyright claim is restricted
// to this file and the depending files "main.h","cforth.c","cforth.h",
// "usb-otg-ctrlbd.c", "dmx.c" and "globaldefines.h"
// Other depending material is published, copyrighted and licensed
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f4xx.h"
#include "globaldefines.h"
#include "main.h"

int main(void)    // ********************************************************
// **************************************************************************
{
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN; // Enable Port A clock
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOBEN; // Enable Port B clock
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOCEN; // Enable Port C clock
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIODEN; // Enable Port D clock

    GPIOA->MODER = 0x00000000;      //primarily all input
    GPIOB->MODER = 0x00000000;      //primarily all input
    GPIOC->MODER = 0x00000000;      //primarily all input
    GPIOA->PUPDR = 0xA402AAAA;      //primarily all inputs pulldown,
                                    //except PA9,10,11,12,13,14
    GPIOB->PUPDR = 0xAAAAAAAA;      //primarily all inputs pulldown
    GPIOC->PUPDR = 0xAAAAAAAA;		//primarily all inputs pulldown,

#ifndef BLACKPILL
  #ifdef STLINK_CUTOFF
	GPIOB->PUPDR &= ~(GPIO_PUPDR_PUPDR2); //PB2 is Vusb bridge,must be HiZ
  #endif
//the DIP switch on the veroboard is not used by CForth
//but it can be used as I/O input (PC2, PC3)
//the green part of color LED on the veroboard is CForth front panel LED (PC0)
//the red part LED (PC1) can be used as user output
    GPIOD->MODER = 0x00000000;	//primarily all input (physically only PD2)
    GPIOD->PUPDR = 0x00000020;	//primarily PD2 pullDown (only existing one)
#endif


    SetSysClock();

//    GPIOA->MODER &= ~(GPIO_MODER_MODER7);
//    GPIOA->MODER |= GPIO_MODER_MODER7_0;	//set PA7(SYSCLCK test) as output

#ifdef BLACKPILL
//pin config repeated specifically:
	GPIOB->PUPDR &= ~(GPIO_PUPDR_PUPDR15);	//pull-down off, wasts current
    GPIOB->MODER &= ~(GPIO_MODER_MODER15);
    GPIOB->MODER |= GPIO_MODER_MODER15_0 ;  	//set PortB15 (front LED) as output
    GPIOB->BSRR =  GPIO_BSRR_BS_15;        	//PB15 (user LED) ON
//PC13 is blue LED on BLACKPILL pcb, repeat special config.
	GPIOC->PUPDR &= ~(GPIO_PUPDR_PUPDR13);		//HiZ
	GPIOC->PUPDR |= GPIO_PUPDR_PUPDR13_1;		//pull-down gives smart blue
#else
  #ifdef STLINK_CUTOFF
	GPIOC->PUPDR &= ~(GPIO_PUPDR_PUPDR0);	//pull-down off, wasts current
    GPIOC->MODER &= ~(GPIO_MODER_MODER0);
    GPIOC->MODER |= GPIO_MODER_MODER0_0 ;//set PC0(green front LED) as output
    GPIOC->BSRR =  GPIO_BSRR_BS_0;        	//PC0 (front LED) ON
  #else
//pin config repeated specifically:
	GPIOA->PUPDR &= ~(GPIO_PUPDR_PUPDR5);	//pull-down off, wasts current
    GPIOA->MODER &= ~(GPIO_MODER_MODER5);
    GPIOA->MODER |= GPIO_MODER_MODER5_0 ;  	//set PortA5 (user LED) as output
    GPIOA->BSRR =  GPIO_BSRR_BS_5;        	//PA5 (user LED) ON
  #endif
#endif
    gSYSCOUNT = 50;
    gLEDCOUNT = 0;
    gDELAYCOUNT = 0;
    gTIMEOUT = 0;
    gProject = 0;
	gfRandom = 314159265;		//seed for xorshift32 algorithm
	gfNumBase = DEFAULT_NUMBASE;
	gfTempNumBase = DEFAULT_NUMBASE;
    gBaud = DEFAULT_BAUD;
	gSplit = 0; //formally needed for compiler/linker in usb-otg-ctrlbd.c
	gLedBusy = 2;
#ifndef ORIG_NUCLEO
    gTempBaud = DEFAULT_BAUD;
    gVid = DEFAULT_VID;
    gPid = DEFAULT_PID;
    guDeviceClass = CDC_CLASS; //seems to be necessary
    guConfigured = 0;
    gUsbChecked = 1;
	gSplit = 0;					//USB and USART in parallel as terminal
	gBaudSet = 0;
	gChannel = 0;
	gMIDI_HEAD = 0;
	gMIDI_TAIL = 0;
  #ifdef DMX
	gdDmxMax = 512;
  #endif
#endif

    SysTick_Config(SystemCoreClock/1000);

    msDelay(100);

//start Independent Watchdog  **************
    IWDG->KR = 0x00005555;
    IWDG->RLR = 0x0000FFFF;
    msDelay(2);
    IWDG->PR = 0x00000002;      //resulting timeout ca. 1-2 seconds
    msDelay(2);
    IWDG->KR = 0x0000CCCC;      //start Independent Watchdog

//init I/O state  *************************
    popFlashSector3();		//if ORIG_NUCLEO:contains some dummies
 //init communication  ********************

#ifdef BLACKPILL
    initUsart1();
#else
  #ifdef STLINK_CUTOFF
    initUsart1();
  #else						//original, un-modified Nucleo board assumed
    initUsart2();
  #endif

#endif
    msDelay(5);

//popFlashSector3();		//uncomment sometimes makes sense for debugging

#ifdef DMX
	startDmxTx();
	DmxHeader();	
	msDelay(2);
	if (gdBootScene)  {
		op_FLUSH();
		gfSP++;
		*gfSP = gdBootScene;
		op_DMXLOAD();
	}
#endif

#ifndef ORIG_NUCLEO
	sendB(CRR);
	sendModel();
    guConfigured = 0;
  #ifdef BLACKPILL
    if ((gVid != 0) && (gPid != 0)) startUSB();
  #else
    if ((GPIOA->IDR&GPIO_IDR_IDR_9) && (gVid != 0) && (gPid != 0)) startUSB();
  #endif
	else  {
        sendB('n');
        sendB('o');
        sendB(' ');
        sendB('U');
        sendB('S');
        sendB('B');
		sendB(CRR);
    }
#else
//file "usb-otg.c"  is included for compiler reasons, but not used in this case
#endif

    popFlashProject();				//load complete project now

    while(1) {
	quit();
	query();
	if (gfTibMax == 0) continue; 
	eval();
    }
//should never been reached, but...
    msDelay(100);
    while(1);						//force cold reset by watchdog
}

//##########################################################################

// Misc Low Level Routines
//************************

void sendModel(void)  //  ****************************************************
//  **************************************************************************
{
#ifdef BLACKPILL
const uint8_t typeStr [] = "CForth for STM32F411 BlackPill v";
#else
const uint8_t typeStr [] = "CForth for STM32 Nucleo-F411 v";
#endif
const uint8_t copyStr [] = "  CPU=84MHz  (c)2020-25 W.Schemmert";

	sendString(typeStr);
	sendTB(HARDWARE_VERSION + 0x30);
	sendTB('.');
	sendNS32(REVNUM);
	sendString(copyStr);
	sendTB(CRR);
}


void SetSysClock(void)
// *****************************************************************************
//             PLL (clocked by HSE) used as System clock source                
// *****************************************************************************
{
uint32_t StartUpCounter = 0, HSEStatus = 0;

//PLL Parameters
#ifdef BLACKPILL
#define PLL_M      25        //changed for project-->1MHz
#define PLL_N      336       //changed for project-->336MHz
#else
#define PLL_M      4        //changed for project-->2MHz
#define PLL_N      168       //changed for project-->336MHz
#endif
//	SYSCLK = PLL_VCO / PLL_P 
#define PLL_P      1        //changed for project-->div./.4 = 84MHz

//USB OTG FS, SDIO and RNG Clock =  PLL_VCO / PLLQ
#define PLL_Q      7        //48MHz changed for project

#define STARTUP_TIMEOUT   0x50000 //Time out for HSE start up

//Select regulator voltage output Scale 1 mode, System frequency up to 84 MHz 
    RCC->APB1ENR |= RCC_APB1ENR_PWREN;
    PWR->CR |= 0x8000;				//0x8000->VOS=10 up to 84 MHz 

// Enable HSE
	RCC->CR |= ((uint32_t)RCC_CR_HSEON);
	RCC->CR &= ~((uint32_t)RCC_CR_HSION);

//Wait till HSE is ready and if Time out is reached exit
  do
  {
    HSEStatus = RCC->CR & RCC_CR_HSERDY;
    StartUpCounter++;
  } while((HSEStatus == 0) && (StartUpCounter != HSE_STARTUP_TIMEOUT));

  if ((RCC->CR & RCC_CR_HSERDY) != RESET)  {

//HCLK = SYSCLK
    RCC->CFGR |= RCC_CFGR_HPRE_DIV1;    //set for project to 84MHz

//PCLK2 = HCLK
    RCC->CFGR |= RCC_CFGR_PPRE2_DIV1;   //set for project to 84MHz

//PCLK1 = HCLK
    RCC->CFGR |= RCC_CFGR_PPRE1_DIV2;   //set for project to 42MHz(max!)

//Configure the main PLL
    RCC->PLLCFGR = PLL_M | (PLL_N << 6) | (PLL_P << 16) |
                   (RCC_PLLCFGR_PLLSRC_HSE) | (PLL_Q << 24);

//Enable the main PLL
    RCC->CR |= RCC_CR_PLLON;

//Wait till the main PLL is ready
    while((RCC->CR & RCC_CR_PLLRDY) == 0) ;

//Configure Flash prefetch, Instruction cache, Data cache and wait state
    FLASH->ACR = FLASH_ACR_PRFTEN | FLASH_ACR_ICEN |FLASH_ACR_DCEN |
														FLASH_ACR_LATENCY_2WS;

//Select the main PLL as system clock source
    RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_SW));
    RCC->CFGR |= RCC_CFGR_SW_PLL;

// Wait till the main PLL is used as system clock source
    while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS ) != RCC_CFGR_SWS_PLL);

//    SystemCoreClockUpdate();
	SystemCoreClock = 84000000;
/*
		//possibly needed for SYSCLK freq test:
//activate MCO_1 PLL/5 out
	RCC->CFGR |= (7 << 24) | (3 << 21);	
//set PA8 in alternate function mode
    GPIOA->MODER &= ~(GPIO_MODER_MODER8);
    GPIOA->MODER |= GPIO_MODER_MODER8_1;
//MCO_1 is AF0 on PA8
    GPIOA->AFR[1] &= 0xFFFFFFF0;
    GPIOA->AFR[1] |= 0x00000000;
*/
  }

}


void LED_Off(void)  // ******************************************************
// built as subroutine to supply service to "usb-fs-431.c" hardware independent
// **************************************************************************
{
	if (gLedBusy > 1)  {
#ifdef BLACKPILL
		GPIOB->BSRR = GPIO_BSRR_BR_15; // clear bit PB15,LED OFF
#else
  #ifdef STLINK_CUTOFF
		GPIOC->BSRR = GPIO_BSRR_BR_0; // clear PC0, green front LED OFF
  #else
		GPIOA->BSRR = GPIO_BSRR_BR_5; // clear bit PA5,LED OFF
  #endif
#endif
		gLEDCOUNT = 4; //ca. 300 milliseconds
	}
}


void msDelay(uint32_t count)   // *******************************************
//blocking delay by 'count' milliseconds
// **************************************************************************
{
    gDELAYCOUNT = count;
	while(gDELAYCOUNT)  servSystem();
}


void wait5us(void)   // *****************************************************
// blocking delay ca 5us
// *************************************************************************
{
uint32_t countdown;

	countdown = 70;				//approx.
    while (countdown)   {
        IWDG->KR = 0x0000AAAA;  //timeout ca. 1-2 seconds
        countdown-- ;
    }
}


void servSystem(void)   // ************************************************
//not very time critical but permanently repeated functions for system control
//exclusively called in pullUsartRxBuf and pullUsbRxBuf
//this way it is regularily served when idle waiting for external input
//but watchdog resets when it hangs somewhere else
//additionally slowed down by SysTick to 100 /20 /1 times/second 
// ****************************************************************************
{
static uint32_t secCount, checkCount;
const uint8_t plugStr [] = "USB plugged ... " ;
const uint8_t unplugStr [] = "USB unplugged" ;

#ifdef DMX
	if (gfDmxBusy)  {
		if (USART2->SR & USART_SR_TXE) {
			USART2->DR = gdDMXBUF[gdDmxTxIndex];
// the TXE bit is cleared automatically when TDR is written
			gdDmxTxIndex++;
			if (gdDmxTxIndex >= gdDmxMax)  DmxHeader() ;
		}


//lower speed services:  **********************************

		if (gSYSCOUNT)  return;		//countdown in SysTick_Handler()
//else if (gSYSCOUNT==0) restart 100Hz service interval
		gSYSCOUNT = 10;
	   IWDG->KR = 0x0000AAAA;		//feed watchdog,timeout ca.1-2seconds

//handle DMX faders
		if (gdFaderBusy)  fadeProcess();
	}
	else  {
		if (gSYSCOUNT)  return;		//countdown in SysTick_Handler()
//else if (gSYSCOUNT==0) restart 100Hz service interval
		gSYSCOUNT = 10;
	   IWDG->KR = 0x0000AAAA;		//feed watchdog,timeout ca.1-2seconds
	}
	checkCount++;
	if (checkCount == 5)  {			//from here 20Hz interval
		checkCount = 0;
#else
		if (gSYSCOUNT)  return;		//countdown in SysTick_Handler()
//else if (gSYSCOUNT==0) restart 20Hz service interval
		gSYSCOUNT = 50;
//feed watchdog
		IWDG->KR = 0x0000AAAA;  //timeout ca. 1-2 seconds
#endif  //DMX or not

//LED control
#ifdef BLACKPILL
		if (gLEDCOUNT)  {
			gLEDCOUNT--;
			if (gLEDCOUNT==0)  {
				if (gLedBusy >= 2)  GPIOB->BSRR =  GPIO_BSRR_BS_15;
													//PB15 (frontLED) ON again
			}
		}
//USB mode control
		secCount++;
		if (secCount >= 20)  {  		//restart 1 second service interval
			secCount = 0;

			if (guConfigured == 0) {
				if (gUsbChecked == 0) {
					sendString(plugStr);
					startUSB();		//USB plugged, enforce reset by Watchdog
				}
			}
			else  {
				if ((gSplit == 4) && ((guDeviceClass) == CDC_CLASS))  {
					stopUSB();
					guDeviceClass = MIDI_CLASS;
					msDelay(500);
					startUSB();
				}
				if ((gSplit < 4) && ((guDeviceClass)== MIDI_CLASS))  {
					stopUSB();
					guDeviceClass = CDC_CLASS;
					msDelay(500);
					startUSB();
				}
			}
		}
#endif
#ifdef STLINK_CUTOFF
		if (gLEDCOUNT)   {
			gLEDCOUNT--;
			if (gLEDCOUNT==0)  {
				if (gLedBusy >= 2)  GPIOC->BSRR = GPIO_BSRR_BS_0; 
											//PC0 (green front LED) ON again
			}
		}
//USB mode control
		secCount++;
		if (secCount >= 20)  {  		//restart 1 second service interval
			secCount = 0;
//USB mode control
			if (guConfigured == 0) {
				if ((GPIOA->IDR & GPIO_IDR_IDR_9) && (gUsbChecked == 0)) {
					sendString(plugStr);
					startUSB();       	//USB plugged, enforce reset by Watchdog
				}
			}
			else  {
				if ((GPIOA->IDR & GPIO_IDR_IDR_9) == 0)  {
					sendString(unplugStr);
					sendB(CRR);
					stopUSB();         	//USB un-plugged, deactivate peripheral
					gUsbChecked = 0;
				}
				else   {                //USB plugged
					if ((gSplit == 4) && ((guDeviceClass) == CDC_CLASS))  {
						stopUSB();
						guDeviceClass = MIDI_CLASS;
						msDelay(500);
						startUSB();
					}
					if ((gSplit < 4) && ((guDeviceClass)== MIDI_CLASS))  {
						stopUSB();
						guDeviceClass = CDC_CLASS;
						msDelay(500);
						startUSB();
					}
				}
			}
		}
#endif
#ifdef ORIG_NUCLEO
		if (gLEDCOUNT)   {
			gLEDCOUNT--;
			if (gLEDCOUNT==0)  {
				if (gLedBusy >= 2)  GPIOA->BSRR =  GPIO_BSRR_BS_5; 
													//PA5 (User LED) ON again
			}
		}
#endif
#ifdef DMX
	}
#endif
}

#ifndef ORIG_NUCLEO
void startUSB(void)  // ******************************************************
// ***************************************************************************
{
uint32_t oldTimeout;
const uint8_t cdcClassStr [] = "USB as virtual COM =" ;
const uint8_t midiClassStr [] = "USB as MIDI interface =" ;

    gUsbChecked = 1;
    if (gSplit == 4) guDeviceClass = MIDI_CLASS; //double safe init
    else guDeviceClass = CDC_CLASS;
    initUSB();
    gTIMEOUT = 3000;
	oldTimeout = 3000;
	while (gTIMEOUT)  {
		IWDG->KR = 0x0000AAAA;		//feed watchdog, timeout ca.0.5c
		if (gTIMEOUT != oldTimeout)  {	//check once per ms
			oldTimeout = gTIMEOUT;
			if ((gTIMEOUT & 0x00000003FF) == 0x3FF)  sendTB('.');
		}
        if (guConfigured)  break;
    }
    if (guDeviceClass == MIDI_CLASS)  sendString(midiClassStr);
    else  sendString(cdcClassStr);	
    if (guConfigured)  {
        sendTB('o');
        sendTB('k');
    }
    else {
        sendB('e');
        sendB('r');
        sendB('r');
        sendB('o');
        sendB('r');
    }
   sendTB(CRR);
    msDelay(200);
}
#endif	//STLINK_CUTOFF

void popFlashSector3(void)   // **********************************************
//read project independent system parameters
// **************************************************************************
{
uint32_t * flashPtr;
uint32_t test, vid, pid;
uint32_t *sramPtr;

	emptyForth();           //init default system paramters

    flashPtr = (uint32_t *)SECTOR3;

    test = ~(*flashPtr);//virgin test cannot be==0 if project was saved in Flash
    flashPtr++;
    vid =  ~(*flashPtr);    //cannot be==0 if any project was saved in Flash
    flashPtr++;
    pid =  ~(*flashPtr);    //cannot be==0 if any project was saved in Flash
    flashPtr++;

#ifndef ORIG_NUCLEO
    if ((test | vid | vid)==0) {//Flash is virgin, load default empty project
		gBaud = DEFAULT_BAUD;	//as precaution, should be already set
		gTempBaud = DEFAULT_BAUD;
		return;					//defaults already init
    }
    gVid = vid;
    gPid = pid;
	test =  ~(*flashPtr);			   //gBaud gfNumBase order changed 28Dec24
	flashPtr++;
	gBaud = (test>>16) & 0x0000007F;	//baud rate is project independent
	if (gBaud == 0) gBaud = DEFAULT_BAUD;
	gTempBaud = gBaud;
	gfNumBase = test & 0x0000001F;
	if ((gfNumBase!=10)&&(gfNumBase!=16)) gfNumBase = DEFAULT_NUMBASE;
	gChannel = ~(*flashPtr);    //MIDI channel is project independent
	flashPtr++;
#else
    if ((test | vid | vid)==0) {//Flash is virgin, load default empty project
		return;					//defaults already init
    }
	test =  ~(*flashPtr);
	flashPtr++;
	gfNumBase = test  & 0x0000001F;
	if ((gfNumBase!=10)&&(gfNumBase!=16)) gfNumBase = DEFAULT_NUMBASE;
	flashPtr++;
#endif	//ORIG_NUCLEO or not

    gProject =  ~(*flashPtr);   //number of project which is loaded by default
								//gProject=0 modifies Empty Project
	flashPtr++;
#ifdef DMX
		gdFadeTime = ~*flashPtr;
		flashPtr++;
		gdBootScene = ~*flashPtr;
		flashPtr++;
#endif
}


uint32_t pushFlashSector3(void)   //******************************************
//save project independent system parameters permanently
// **************************************************************************
{
uint32_t * flashPtr;
uint32_t k, word;

//unlock flash
#define KEY1    0x45670123      //mandatory unlock key defined by STM
#define KEY2    0xCDEF89AB

    while ((FLASH->SR &FLASH_SR_BSY) !=0); //wait until no operation is going on
    if ((FLASH->CR & FLASH_CR_LOCK) != 0) {
	FLASH->KEYR = KEY1;
	FLASH->KEYR = KEY2;
    }
    else return 0;
 
//feed watchdog
    IWDG->KR = 0x0000AAAA;          //timeout ca. 1-2 seconds

//erase Sector3
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;  //wait until flash is not busy
    FLASH->CR = (FLASH_CR_PSIZE_1 | (3<<3) | FLASH_CR_EOPIE | FLASH_CR_SER);
			//enable Erase Sector3, EOPIE is set formally to get SR_EOP set
    FLASH->CR |= FLASH_CR_STRT;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) !=0)  FLASH->SR |= FLASH_SR_EOP;
    else  return 0;
    FLASH->CR &= ~FLASH_CR_SER;

//write mirror into Sector3:
//word-wide written into flash
//only write USB Vid/Pid, gBaud, gProject and DMX globsals into page23
    flashPtr = (uint32_t *)SECTOR3;

    FLASH->CR |= FLASH_CR_PG;
#ifndef ORIG_NUCLEO
    *flashPtr = 0;                 //for flash virgin test
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = ~(gVid & 0x0000FFFF);
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = ~(gPid & 0x0000FFFF);
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

	*flashPtr = ~((gBaud <<16) | gfNumBase);		//order changed 28Dec24
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = ~gChannel;
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = ~gProject;
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

#else		//dummy entries for compatibility of Flash with STLINK_CUTOFF
    *flashPtr = 0;                 //for flash virgin test
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = ~(DEFAULT_VID & 0x0000FFFF);
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = ~(DEFAULT_PID & 0x0000FFFF);
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

	*flashPtr = ~(DEFAULT_BAUD | (gfNumBase << 16));
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = 0xFFFFFFFF;		//dummy for gChannel
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = ~gProject;
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

#endif		//ORIG_NUCLEO or not

#ifdef DMX
	*flashPtr = ~gdFadeTime;
	flashPtr++;
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;
	*flashPtr = ~gdBootScene;
	flashPtr++;
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;
#endif

//terminate
    FLASH->CR &= ~FLASH_CR_PG;
//lock flash
    FLASH->CR |= FLASH_CR_LOCK;
    msDelay(10);
    return  1;
}


void popFlashProject(void)   // *********************************************
//first read some project specific globals
//next read 1 page to gfUserHeader[] and  to gfTCode
// **************************************************************************
{
uint32_t k, m, n;
uint32_t * flashPtr;
uint32_t word, test;

//emptyForth() already updated in popFlashSector3() (default system params)
//details of selected project are stored in Sector4 or 5 or 6 or 7
	if (gProject)  {
		if (gProject == 1) flashPtr = (uint32_t *)SECTOR4;
		else if (gProject == 2) flashPtr = (uint32_t *)SECTOR5;
		else if (gProject == 3) flashPtr = (uint32_t *)SECTOR6;
		else  flashPtr = (uint32_t *)SECTOR7;

//first 32 bytes with system & peripheral config parameters
		test =  ~(*flashPtr);	//cannot be==0 if any project saved in Flash
		flashPtr++;
		if (test == 0) {		//project Flash is virgin
			emptyForth();		//implies gProject=0 -> empty project
		}
		else  {
#ifndef ORIG_NUCLEO
			gMIDI_HEAD = 0;			//flush MIDI buffer
			gMIDI_TAIL = 0;
#endif
			gfNumBase= ~(*flashPtr);	//overrides Sector3 global setup
			flashPtr++;
			if ((gfNumBase!=10)&&(gfNumBase!=16)) gfNumBase = DEFAULT_NUMBASE;
			gfTempNumBase = gfNumBase;

			gfAuto = ~(*flashPtr);
			flashPtr ++;

			word = ~(*flashPtr);
			flashPtr ++;
			gfSpiMode = (word >> 24) & 0x00000003;
			gfSpiBits = (word >> 16) & 0x00000003F;
			gfSpiClock = word & 0x00000FFFF;

			gfFreq12 = ~(*flashPtr);
			flashPtr ++;

			k = ~(*flashPtr);
			flashPtr ++;
			gfPWM1 = (int32_t)(k >> 16);
			if ((gfPWM1 == 0) || (gfPWM1 == 0x0000FFFF)) gfPWM1 = -1;
			gfPWM2 = (int32_t)(k & 0x0000FFFF);
			if ((gfPWM2 == 0) || (gfPWM2 == 0x0000FFFF)) gfPWM2 = -1;
			if (gfFreq12)  {
				subFREQ12();
				subPWM1();
				subPWM2();
			}
			gfFreq3 = ~(*flashPtr);
			flashPtr ++;

			k = ~(*flashPtr);
			flashPtr ++;
			gfPWM3 = (int32_t)(k >> 16);
			if ((gfPWM3 == 0) || (gfPWM3 == 0x0000FFFF)) gfPWM3 = -1;
			if (gfFreq3)  {
				subFREQ3();
				subPWM3();
			}
#ifdef DMX
			gdFadeTime = ~(*flashPtr);	//overrides Sector3 global setup
			flashPtr ++;
			gdBootScene = ~(*flashPtr);
			flashPtr ++;
#else	
			flashPtr += 2;			//dummy to maintain compatibility
#endif
			for (k = 0;k < MAXUSER;k++)  {
									//reads 168*24 words=4032bytes +40=4072
				n = 0;
				for (m = 0; m < 4; m++) {
					word = ~(*flashPtr);
					flashPtr++;
					gfUserHeader[k].name[n]= word & 0x000000FF;
					n++;
					gfUserHeader[k].name[n]= (word >> 8) & 0x000000FF;
					n++;
					gfUserHeader[k].name[n]= (word >> 16) & 0x000000FF;
					n++;
					gfUserHeader[k].name[n]= (word >> 24) & 0x000000FF;
					n++;
				}
				gfUserHeader[k].toCompile= ~(*flashPtr);
				flashPtr ++;
				gfUserHeader[k].attributes = ~(*flashPtr);
				flashPtr ++;
			}

			for (k = 0;k < MAXCODE;k++)  {	//reads 0xC00 words=12288 bytes
				gfTCode[k] = ~(*flashPtr);	//from upper part of project page
				flashPtr ++;
			}
		}
	}

//from here if(gProject) or virgin  or not:
	wait5us();
    op_ABORT();

//emergency brake to avoid AUTO deadlock
#ifdef BLACKPILL
	if ((GPIOA->IDR & 0x0400)==0) gfAuto =0; //if PA10 (RS232 RX)is LOW
#else
	if ((GPIOC->IDR & 0x2000)==0) gfAuto =0; //if PC13 (user button)is LOW
#endif

    op_QUEST();
    if (gfAuto) {
        gfParsed = gfAuto - 1;	//-1 to distinguish from empty AUTOEXE == 0
        interpretUserToken();
    }
}


uint32_t pushFlashProject(void)   //*****************************************
// **************************************************************************
{
uint32_t k, m, n, sector;
uint32_t * flashPtr;
uint32_t word[6];

//unlock flash
#define KEY1    0x45670123      //mandatory unlock key defined by STM
#define KEY2    0xCDEF89AB

    if (gProject == 0) return 1;   //=default empty project, don't save

    while ((FLASH->SR &FLASH_SR_BSY) !=0); //wait until no operation is going on
    if ((FLASH->CR & FLASH_CR_LOCK) != 0) {
	FLASH->KEYR = KEY1;
	FLASH->KEYR = KEY2;
    }

//feed watchdog
    IWDG->KR = 0x0000AAAA;          //timeout ca. 1-2 seconds

    sector = gProject + 3;			//project no.0 is empty, not treated here
                                    //project no.1 starts with Sector4
//erase Sector4 or 5 or 6 or 7
    while ((FLASH->SR & FLASH_SR_BSY) != 0)  IWDG->KR = 0x0000AAAA; 
									//wait until flash is not busy

    FLASH->CR = (FLASH_CR_PSIZE_1| (sector<<3)| FLASH_CR_EOPIE |FLASH_CR_SER);
									//EOPIE is set formally to get SR_EOP set
    FLASH->CR |= FLASH_CR_STRT;
    while ((FLASH->SR & FLASH_SR_BSY) != 0)  IWDG->KR = 0x0000AAAA;

    if ((FLASH->SR & FLASH_SR_EOP) !=0)  FLASH->SR |= FLASH_SR_EOP;
    else  return 0;
    FLASH->CR &= ~FLASH_CR_SER;

//first write virgin test, gfNumBase, gfAuto, SPI and FREQ, PWM parameters
//word-wide written into flash
	if (gProject == 1) flashPtr = (uint32_t *)SECTOR4;
	else if (gProject == 2) flashPtr = (uint32_t *)SECTOR5;
	else if (gProject == 3) flashPtr = (uint32_t *)SECTOR6;
	else  flashPtr = (uint32_t *)SECTOR7;

    FLASH->CR |= FLASH_CR_PG;
//first 40 bytes with system & peripheral config parameters
    *flashPtr = 0x00000000;			//virgin flash test
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = ~gfNumBase;			//overrides Sector3 global setup
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = ~gfAuto;
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = (~((gfSpiMode << 24) | (gfSpiBits << 16 ) |gfSpiClock));
	flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = ~gfFreq12;
	flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = ~((((uint32_t)gfPWM1 << 16) & 0xFFFF0000) | 
											((uint32_t)gfPWM2 & 0x0000FFFF));
	flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;
	
    *flashPtr = ~gfFreq3;
	flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

    *flashPtr = ~((((uint32_t)gfPWM3 << 16) & 0xFFFF0000));
	flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;
#ifdef DMX
	*flashPtr = ~(gdFadeTime);		//overrides Sector3 global setup
	flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;

	*flashPtr = ~(gdBootScene);		//overrides Sector3 global setup
	flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	else  return 0;
#else
	*flashPtr = 0xFFFFFFFF;		//dummies for code compatibility
	flashPtr++;
	*flashPtr = 0xFFFFFFFF;
	flashPtr++;
#endif

//write gfUserHeader[] into sector
//word-wide written into flash
    for (k = 0;k < MAXUSER; k++) {	//MAXUSER = 168 ->4032bytes+32=4064
        n = 0;
        for (m = 0; m < 4; m++) {
            word[m] = gfUserHeader[k].name[n];
            n++;
            word[m] |= gfUserHeader[k].name[n] << 8;
            n++;
            word[m] |= gfUserHeader[k].name[n] << 16;
            n++;
            word[m] |= gfUserHeader[k].name[n] << 24;
            n++;
        }
        word[4] = gfUserHeader[k].toCompile;
        word[5] = gfUserHeader[k].attributes;

        for (m = 0; m < 6; m++)  {
//word-wide written into flash
            *flashPtr = ~word[m]; //~for better compatibility with empty SRAM
            flashPtr++;
            while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
			if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
			else  return 0;
        }
		if ((k & 0x3FF) == 0)   {
			IWDG->KR = 0x0000AAAA;  //feed watchdog
			sendTB('.');
		}
    }

//write  gTCode into upper space of Sector 4 or 5 or 6 or 7
//word-wide written into flash
    for (k = 0; k < 0xC00; k++)  {
		*flashPtr = ~gfTCode[k]; //~for better compatibility with empty SRAM
		flashPtr++;
		while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
		if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
		else  return 0;

		if ((k & 0x3FF) == 0)  {
			IWDG->KR = 0x0000AAAA;  //feed watchdog
			sendTB('.');
		}
    }

//terminate
    FLASH->CR &= ~FLASH_CR_PG;
//lock flash
    FLASH->CR |= FLASH_CR_LOCK;
    msDelay(10);
    return 1;
}

#ifdef DMX
uint32_t pushDmxFlash(void)   //*********************************************
// **************************************************************************
{
uint32_t k;
uint32_t * flashPtr;
uint32_t * dmxPtr;

//unlock flash
#define KEY1    0x45670123      //mandatory unlock key defined by STM
#define KEY2    0xCDEF89AB

    while ((FLASH->SR &FLASH_SR_BSY) !=0); //wait until no operation is going on
    if ((FLASH->CR & FLASH_CR_LOCK) != 0) {
	FLASH->KEYR = KEY1;
	FLASH->KEYR = KEY2;
    }
//feed watchdog
    IWDG->KR = 0x0000AAAA;          //timeout ca. 1-2 seconds

//erase SECTOR7
    while ((FLASH->SR & FLASH_SR_BSY) != 0)  IWDG->KR = 0x0000AAAA;  
											//wait until flash is not busy
    FLASH->CR = (FLASH_CR_PSIZE_1| (7<<3)| FLASH_CR_EOPIE |FLASH_CR_SER);
			//					EOPIE is set formally to get SR_EOP set
    FLASH->CR |= FLASH_CR_STRT;
    while ((FLASH->SR & FLASH_SR_BSY) != 0)  IWDG->KR = 0x0000AAAA;

    if ((FLASH->SR & FLASH_SR_EOP) !=0)  FLASH->SR |= FLASH_SR_EOP;
    else  return 0;
    FLASH->CR &= ~FLASH_CR_SER;

//word-wide written into flash
	flashPtr = (uint32_t *)SECTOR7;
    FLASH->CR |= FLASH_CR_PG;

//this microcontroller allows word-wide write in flash
	dmxPtr = (uint32_t *)gfTempBuf;
    for (k = 0; k < 12288; k ++)  {		//48kBytes / 4
		*flashPtr = ~(*dmxPtr);
		flashPtr++;
		dmxPtr++;	
		while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
		if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
		else  return 0;
		if ((k & 0x00000FFF) == 0) {
			sendTB('.');
			IWDG->KR = 0x0000AAAA;	//feed watchdog
		}
    }

//terminate
    FLASH->CR &= ~FLASH_CR_PG;
//lock flash
    FLASH->CR |= FLASH_CR_LOCK;
    msDelay(10);
    return 1;	
}	
#endif //DMX

//###########################################################################

//USART Support
//*************

#ifndef ORIG_NUCLEO
void initUsart1(void) // ****************************************************
// **************************************************************************
{
uint16_t baudset;

	RCC->APB2ENR |=  RCC_APB2ENR_USART1EN;	// USART1 clock enable

#ifdef BLACKPILL
// USART1-TxD is PortA,9
// USART1-RxD is PortA,10
	GPIOA->MODER &= ~(GPIO_MODER_MODER9_0 | GPIO_MODER_MODER10_0);	
											//reset PA9,PA10 as input
	GPIOA->MODER |=(GPIO_MODER_MODER9_1 | GPIO_MODER_MODER10_1);	
											//PA9 and PA10 alternate function
//USART1 is AF7
    GPIOA->AFR[1] &= 0xFFFFF00F;        //PA9 is TX, PA10 is RX
    GPIOA->AFR[1] |= 0x00000770;
#else
// USART1-TxD is PortB,6
// USART1-RxD is PortA,10
	GPIOA->MODER &= ~GPIO_MODER_MODER10_0;	//reset PA10,PB6 as input
	GPIOB->MODER &= ~GPIO_MODER_MODER6_0;
	GPIOA->MODER |= GPIO_MODER_MODER10_1;	//PB6 and PA10 alternate function
	GPIOB->MODER |= GPIO_MODER_MODER6_1;	//10=Alternate function mode

//USART1 is AF7
    GPIOA->AFR[1] &= 0xFFFFF0FF;        //PA10 is RX
    GPIOA->AFR[1] |= 0x00000700;
    GPIOB->AFR[0] &= 0xF0FFFFFF;        //PB6 is TX
    GPIOB->AFR[0] |= 0x07000000;
#endif
    gUSART_RXHEAD = gUSART_RXBUF;
    gUSART_RXTAIL = gUSART_RXBUF;

    switch(gBaud) {
    case 96:
        baudset = 0x222E;       //9600Baud: 546.88 at 84 MHz APB2 Clock;
    case 19:
        baudset = 0x1117;       //19200Baud: 273.44 at 84 MHz APB2 Clock
        break;
    case 31:
        baudset = 0x0A80 ;      //MIDI=31250Baud: 168.0 at 84 MHz APB2 Clock
        break;
    case 38:
        baudset = 0x088B ;      //38400Baud: 136.72 at 84 MHz APB2 Clock
        break;
    case 57:
        baudset = 0x05B2;       //57600Baud: 91.15 at 84 MHz APB2 Clock
        break;
    default:
    case 11:
        baudset = 0x02D9;       //default (virgin flash)
                                //115200Baud: 45.57 at 84 MHz APB2 Clock
    }
    USART1->BRR = baudset;
	USART1->CR2 = 0;			//default:1Stopbit, needs an external inverter
    USART1->CR3 = 0;			//default:no Handshake
    USART1->CR1 = USART_CR1_UE |USART_CR1_RXNEIE |USART_CR1_TE | USART_CR1_RE;
													//8Bit, no Parity, RX Int
	gBaudSet = 1;
    NVIC_EnableIRQ(USART1_IRQn);
       //effectively TX interrupt is enabled when bytes are put into the queue
}


void sendB(uint8_t data) // *************************************************
//send 1 raw byte
// **************************************************************************
{
uint32_t fillstate;

    while(!(USART1->SR & USART_SR_TXE));
    USART1->DR = data;
	if (gLedBusy == 3) LED_Off();

}


void sendTB(uint8_t data) // ************************************************
//send one byte via serial terminal port (USB and/or USART)
//0=USB and USART both terminal, 1+3= USB is terminal, 2+4=USART is terminal
// **************************************************************************
{
    if (gSplit == 0)  {
        sendB(data);
#ifndef USB_DEBUG
        if (guConfigured)  sendUsbByte(data);
#endif
    }
#ifndef USB_DEBUG
    else if (((gSplit == 1) || (gSplit == 3)) && (guConfigured))  {
                            //USB is terminal(virtual COM),USART is serial I/O
        sendUsbByte(data);
    }
#endif
    else  sendB(data);      //USART is terminal, USB is serial or MIDI I/O
}


void sendIOB(uint8_t data) // ************************************************
//send one byte via serial I/O
// **************************************************************************
{
    if ((gSplit == 1) || (gSplit == 3)) { 	    //USB is terminal(virtual COM),
        sendB(data); 						    //USART is serial I/O
    }
#ifndef USB_DEBUG
    else if ((gSplit == 2) && (guConfigured)) { //USART=terminal, USB=serial I/O
        sendUsbByte(data); 
    }
    else if ((gSplit == 4) && (guConfigured))  { //USART=terminal, USB=MIDI I/O
        enCodeMidi(data);
    }
#endif
	else  gfErrors |= ERR_CONTEXT;		//gSplit=0
}

	
void sendNibble(uint8_t data) // ********************************************
//0=USB and USART both terminal, 1= USB is terminal, 2||3=USART is terminal
//this communication is exclusively performed with "terminal"
// **************************************************************************
{
uint8_t asciiNibble;

    asciiNibble = data+0x30;
    if (asciiNibble>0x39) asciiNibble+= 7;
    if (gSplit == 0)  {
        sendB(asciiNibble);
#ifndef USB_DEBUG
        if (guConfigured)  sendUsbByte(asciiNibble);
#endif
    }
#ifndef USB_DEBUG
    else if (((gSplit == 1) || (gSplit == 3)) && (guConfigured))  {
                            //USB is terminal(virtual COM),USART is serial I/O
        sendUsbByte(asciiNibble);
    }
#endif
    else sendB(asciiNibble);
}

#else

void initUsart2(void) // ****************************************************
// USART2-TxD is PortA,2
// USART2-RxD is PortA,3
// **************************************************************************
{
    RCC->APB1ENR |=  RCC_APB1ENR_USART2EN ;   // USART2 clock enable

	GPIOA->MODER &= ~(GPIO_MODER_MODER2_0 | GPIO_MODER_MODER3_0);	
											//pin 2,3  alternate function mode
	GPIOA->MODER |= (GPIO_MODER_MODER2_1 | GPIO_MODER_MODER3_1); 
											//10=Alternate function mode
//USART2 is AF7
    GPIOA->AFR[0] &= 0xFFFF00FF;
    GPIOA->AFR[0] |= 0x00007700;

    gUSART_RXHEAD = gUSART_RXBUF;
    gUSART_RXTAIL = gUSART_RXBUF;

    USART2->BRR = 0x016C;       //default (virgin flash)
                                //115200Baud: 22.8 at 42 MHz APB1 Clock
                          //only option when used together with onboard ST-Link

    USART2->CR2 = 0;			//default:1 Stopbit
    USART2->CR3 = 0;			//default:kein Handshake
    USART2->CR1 = USART_CR1_UE | USART_CR1_RXNEIE |USART_CR1_TE | USART_CR1_RE;
                                //8Bit, no Parity, RX Int
    NVIC_EnableIRQ(USART2_IRQn);
        //effectively TX interrupt is enabled when bytes are put into the queue
}


void sendTB(uint8_t data) // *************************************************
//send 1 raw byte
// **************************************************************************
{
uint32_t fillstate;

    while(!(USART2->SR&USART_SR_TXE));
    USART2->DR = data;
	if (gLedBusy == 3) LED_Off();		//new 04Jan24
}


void sendNibble(uint8_t data) // ********************************************
// **************************************************************************
{
uint8_t asciiNibble;

    asciiNibble = data+0x30;
    if (asciiNibble>0x39) asciiNibble+= 7;
    sendTB(asciiNibble);
}
#endif

void sendString(const uint8_t * str) // *************************************
// **************************************************************************
{
 uint8_t byte;

    while  ((byte = *str++) != 0)  sendTB(byte);
}


void sendX8(uint32_t data) // *********************************************
// send bits 0...7 in hex format (without leading 0x)
// **************************************************************************
{
    uint8_t temp;

    temp= (data&0xF0) >> 4;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendTB(temp);
    temp= data&0x0F;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendTB(temp);
    sendTB(0x20);
}


void sendX32(uint32_t data) // **********************************************
// send 1 word (32bit) in hex format (without leading 0x)
// **************************************************************************
{
uint8_t nibble;
    nibble = (uint8_t) ((data>>28) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>24) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>20) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>16) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>12) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) (data & 0x0000000F);
    sendNibble(nibble);
    sendTB(0x20);

}

void sendNX32(uint32_t data) // **********************************************
// send 1 word (32bit) in hex format (without leading 0x)
// **************************************************************************
{
uint8_t nibble, noLead;
    noLead = 0;
    nibble = (uint8_t) ((data>>28) & 0x0000000F);
    if (nibble)  {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) ((data>>24) & 0x0000000F);
    if (nibble || noLead) {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) ((data>>20) & 0x0000000F);
    if (nibble || noLead) {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) ((data>>16) & 0x0000000F);
     if (nibble || noLead) {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) ((data>>12) & 0x0000000F);
    if (nibble || noLead) {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) ((data>>8) & 0x0000000F);
    if (nibble || noLead) {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) ((data>>4) & 0x0000000F);
    if (nibble || noLead) {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) (data & 0x0000000F);
    sendNibble(nibble);
}


void send0X32(uint32_t data) // **********************************************
// send 1 word (32bit) in hex format (without leading 0x)
// **************************************************************************
{
//	if (data >= 10)  {			//modified 11Jan24
		sendTB('0');
		sendTB('x');
//	}
	sendNX32(data);
}


void sendS32(uint32_t data) // *******************************************
//send signed decimal number string. Leading zeroes are suppressed
//trailing SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
    sendNS32(data);
    sendTB(0x20);
}


 void sendLS32(uint32_t data) // ********************************************
//send signed decimal number string. Leading zeroes are suppressed
//leading SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
    sendTB(0x20);
    sendNS32(data);
}


void sendNS32(uint32_t data) // *******************************************
//send signed decimal number string. Leading zeroes are suppressed
//no leading and trailing SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
uint32_t quotient;

    if (data >= 0x80000000) {
        sendTB('-');
        data = ~data + 1 ;
    }
    if (data < 10) goto ONES;
    if (data < 100) goto TENS;
    if (data < 1000) goto HUNDREDS;
    if (data < 10000) goto THOUSANDS;
    if (data < 100000) goto TH10;
    if (data < 1000000) goto TH100;
    if (data < 10000000) goto MILLIONS;
    if (data < 100000000) goto MIL10;
    if (data < 1000000000) goto MIL100;
    quotient = data/1000000000;
    data = data - quotient*1000000000;
    sendTB((uint8_t)quotient+ 0x30);
MIL100:
    quotient = data/100000000;
    data = data - quotient*100000000;
    sendTB((uint8_t)quotient + 0x30);
MIL10:
    quotient = data/10000000;
    data = data - quotient*10000000;
    sendTB((uint8_t)quotient + 0x30);
MILLIONS:
    quotient = data/1000000;
    data = data - quotient*1000000;
    sendTB((uint8_t)quotient + 0x30);
TH100:
    quotient = data/100000;
    data = data - quotient*100000;
    sendTB((uint8_t)quotient+ 0x30);
TH10:
    quotient = data/10000;
    data = data - quotient*10000;
    sendTB((uint8_t)quotient + 0x30);
THOUSANDS:
    quotient = data/1000;
    data = data - quotient*1000;
    sendTB((uint8_t)quotient + 0x30);
HUNDREDS:
    quotient = data/100;
    data = data - quotient*100;
    sendTB((uint8_t)quotient+ 0x30);
TENS:
    quotient = data/10;
    data = data - quotient*10;
    sendTB((uint8_t)quotient + 0x30);
ONES:
    sendTB((uint8_t)data + 0x30) ;
}

#ifndef ORIG_NUCLEO
uint32_t pullUsartRxBuf(void) // ********************************************
#else
uint32_t pullTermRxBuf(void) // *********************************************
#endif
// pull 1 byte asynchronously from USART reveive buffer
// check first if something received. If yes then set bit 15. If not return 0
// less overhead and more safe than use of a separate "buffer filled" flag
// **************************************************************************
{
uint32_t recv;

	servSystem();   //serve watchdog etc.

    if (gUSART_RXTAIL!=gUSART_RXHEAD)  {
        recv= (uint32_t)((*gUSART_RXTAIL++ & 0x000000FF)|0x00008000);
        if (gUSART_RXTAIL==gUSART_RXBUFTOP) {
                gUSART_RXTAIL=gUSART_RXBUF;
        }
		LED_Off();
        return recv;
    }
    else return 0;
}

#ifndef ORIG_NUCLEO
uint32_t pullTermRxBuf(void)   // *******************************************
// receive byte from serial terminal port with priority order USB > RS232
// **************************************************************************
{
uint32_t recv;

    if (gSplit == 0)  {
        if (guConfigured)  {            //ASART and USB are parallel terminal
            recv = pullUsbRxBuf();
            if (recv) return recv;      //USB has priority over USART
        }
        recv = pullUsartRxBuf();
        return recv;
    }
    else if (((gSplit == 1)||(gSplit == 3))&&(guConfigured)) {//USB is terminal
        recv = pullUsbRxBuf();  //if (guConfigured == 0): no terminal available
        return recv;
    }
    else   {                            //gSplit = 2 or 4:USART is terminal
            recv = pullUsartRxBuf();
            return recv;
    }
    return 0;   //formally, impossible in practice
}


uint32_t pullIORxBuf(void)  // *********************************************
// receive byte from serial terminal port with priority order USB > RS232
// *************************************************************************
{
uint32_t recv;

    switch (gSplit)  {
    case 1:
        recv = pullUsartRxBuf();  //USART is serial I/O (COM)
        return recv;
    case 3:
        recv = pullUsartRxBuf();  //USART is MIDI IN
        if (recv)  MidiScanner(recv & 0x000000FF);
        return 0;               //no further action in "query[]";
    case 2:                     //USB is serial I/O as virtual COM
        if (guConfigured)  {
            recv = pullUsbRxBuf();
            return recv;
        }
        else return 0;          //if(guConfigured==0) no serial I/O available
    case 4:                     //USB is serial I/O as MIDI interface
        if (guConfigured)  {
            recv = pullUsbRxBuf();
            if (recv)  MidiScanner(recv & 0x000000FF);
            return 0;           //no further action in "query[]";
        }
        else return 0;          //if(guConfigured==0) no serial I/O available
    case 0:                     //if gSplit==0: no serial I/O available
    default:
        return  0;
    }
}


void  MidiScanner(uint32_t byte)  // ****************************************
// accepts only MIDI channel as stored in "gChannel", 
// exception: gChannel = 0xFF, then all MIDI channels are accepted
// **************************************************************************
{
static uint32_t status, data1, msgtype;
static uint32_t progress;

    if (byte >= 0x80) {					//new status byte
			if ((byte == 0xF0) || (byte == EOX) || (byte == 0xF4) ||
						(byte == 0xF5) || (byte == 0xF9) ||(byte == 0xFD) ) {
//								kill SysEx(F0),EOX is end of SysEx message,
//								0xF4, 0xF5, 0xF9 and 0xFD are not defined
				progress = 0;			//reset parser
				status = 0;				//drop input until next status byte
				msgtype = 0;
				return;
			}
			if ((byte == 0xF6) || (byte >= 0xF8))  { //1 byte msg
				gMIDIBUF[gMIDI_HEAD] = byte << 16;		//data bytes = 0
				gMIDI_HEAD++;
				if (gMIDI_HEAD == MIDI_BUFLEN) gMIDI_HEAD = 0;
				progress = 0;			//reset parser
				status = 0;				//drop input until next status byte
				msgtype = 0;
				return;
			}
// "normal" MIDI Channel Msg:
			if (((byte & 0x0000000F) != (gChannel - 1)) && (gChannel != 0)) {
				progress = 0;			//reset parser
				status = 0;				//drop input until next status byte
				msgtype = 0;
				return;
			}	//else accepted status byte:
			status = byte;
			msgtype = byte & 0xF0;
			progress = 1;
	}
	else  {								//new data byte
		if (progress == 0)  {	//SysEx or byte out of context, reset parser
			status = 0;
			msgtype = 0;
			data1 = 0;
			return;
		}
		if (progress == 1)  {
			data1 = byte;
			if ((msgtype == 0xC0)||(msgtype == 0xD0) || (status == 0xF1)||
									(status == 0xF3)) { //2 byte msgs
				gMIDIBUF[gMIDI_HEAD] = (status << 16) | data1;
				gMIDI_HEAD++;					//bits 0..7 for easier parsing
				if (gMIDI_HEAD == MIDI_BUFLEN) gMIDI_HEAD = 0;
				return;		//keep progress==1 and gStatus for Running State
			}
			progress = 2;
			return;
		}
		if (progress == 2)  {					//any 3 byte messages
			gMIDIBUF[gMIDI_HEAD] = (status << 16) | (data1 << 8) | byte;
			gMIDI_HEAD++;					//bits 0..7 for easier parsing
			if (gMIDI_HEAD == MIDI_BUFLEN) gMIDI_HEAD = 0;
			progress = 1;   //reset progress and keep status for Running State
		}
	}
}


uint32_t pullMidiBuf(void) // ********************************************
// pull one word asynchronously from MIDIBUF
// check first if something received. If yes then set bit 31. If not return 0
// less overhead and more safe than use of a separate "buffer filled" flag
// **************************************************************************
{
uint32_t recv;

	servSystem();   //serve watchdog etc.

	if (gMIDI_TAIL != gMIDI_HEAD)  {
		recv= gMIDIBUF[gMIDI_TAIL] | 0x80000000;
		gMIDI_TAIL++;
		if (gMIDI_TAIL== MIDI_BUFLEN)  gMIDI_TAIL= 0;
		LED_Off();
		return recv;
	}
	else return 0;
}
#endif

// ##########################################################################
// Interrupt Handlers

void SysTick_Handler(void)  // **********************************************
// **************************************************************************
{
	if (gSYSCOUNT)  gSYSCOUNT--;		//controls servSystem()
    if (gDELAYCOUNT) gDELAYCOUNT--;
    if (gTIMEOUT) gTIMEOUT--;			//to be used in any subroutine
    if (gfMSCOUNT) gfMSCOUNT --;		//used by op_MS,OP_DS
    if (gfTIXCOUNT) gfTIXCOUNT --;		//used by op_TIX,op_WAIT
	if (gfSTEPCOUNT1) gfSTEPCOUNT1--;	//stepper timing
	if (gfSTEPCOUNT2) gfSTEPCOUNT2--;
	if (gfSTEPCOUNT3) gfSTEPCOUNT3--;
	gfRandom ^= gfRandom << 13;				//xorshift32 algorithm/white noise
	gfRandom ^= gfRandom >> 17;
	gfRandom ^= gfRandom << 5;	
//SYSCLCK test:
//	if (GPIOA->ODR & mask7)  GPIOA->BSRR =  GPIO_BSRR_BR_7;
//	else  GPIOA->BSRR =  GPIO_BSRR_BS_7;
}

#ifndef ORIG_NUCLEO
void USART1_IRQHandler(void)    // *****************************************
// *************************************************************************
{
uint32_t recv;

	if (USART1->SR & USART_SR_RXNE)  {
		recv = USART1->DR & 0x00FF;
		if ((gSplit != 1) && (gSplit != 3)) {
							//	USART is terminal and MiniDmx is not active
			if (recv == CTRL_C)  {
				gfTermCtrlBC = CTRL_C;
				return;
			}
			if (recv != CTRL_B)  {
				*gUSART_RXHEAD = (uint8_t)recv;
				gUSART_RXHEAD++;
				if (gUSART_RXHEAD==gUSART_RXBUFTOP)gUSART_RXHEAD=gUSART_RXBUF;
			}
			else  op_BAKOP();
		}
		else {									//no special char handling
			*gUSART_RXHEAD = (uint8_t)recv;
			gUSART_RXHEAD++;
			if (gUSART_RXHEAD==gUSART_RXBUFTOP)  gUSART_RXHEAD=gUSART_RXBUF;
		}
	}
}

#else

void USART2_IRQHandler(void)    // *****************************************
// *************************************************************************
{
uint32_t test;

    if (USART2->SR & USART_SR_RXNE)  {
		test = USART2->DR & 0x00FF;
//		sendTB((uint8_t)test);
		if (test == CTRL_C)  {
			gfTermCtrlBC = CTRL_C;
			return;
		}
		if (test != CTRL_B)  {
			*gUSART_RXHEAD = (uint8_t)test;
			gUSART_RXHEAD++;
			if (gUSART_RXHEAD==gUSART_RXBUFTOP)  gUSART_RXHEAD=gUSART_RXBUF;
		}
		else  op_BAKOP();
	}
}
#endif
